""" This module is loaded instead of ufl when executing .ufl files.
So, this module contains all our extensions and monkey patches to
UFL.
"""

import ufl
from ufl import *
from ufl.split_functions import split

class TrialFunction(ufl.Coefficient):
    """ A coefficient that always takes the reserved index 0 """
    def __init__(self, element, count=None):
        if count and count is not 0:
            raise ValueError("The trial function must be the coefficient of index 0 in uflpdelab")
        ufl.Coefficient.__init__(self, element, count=0)

class Coefficient(ufl.Coefficient):
    """ A coefficient that honors the reserved index 0. """
    def __init__(self, element, count=None):
        if count and count is 0:
            raise ValueError("The coefficient of index 0 is reserved for the trial function in uflpdelab")
        if not count and ufl.Coefficient._globalcount is 0:
            count = 1
        ufl.Coefficient.__init__(self, element, count)

def Coefficients(element):
    return split(Coefficient(element))

def TrialFunctions(element):
    return split(TrialFunction(element))
